<?php

namespace App\Entity;

use App\Repository\TagRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * A tag
 *
 * Tags are by user and a “materialized path” tree structure
 *
 * @link https://github.com/doctrine-extensions/DoctrineExtensions/blob/main/doc/tree.md#materialized-path
 */
#[ORM\Entity(repositoryClass: TagRepository::class)]
#[Gedmo\Tree(type: 'materializedPath')]
class Tag {
    use TEntityBase;

    /**
     * @var User|null The owner
     */
    #[ORM\ManyToOne(inversedBy: 'tags')]
    #[ORM\JoinColumn(nullable: false, onDelete: 'CASCADE')]
    #[Assert\Valid]
    private ?User $owner = null;

    /**
     * @var string|null The name
     */
    #[ORM\Column(length: 50)]
    #[Gedmo\TreePathSource]
    #[Assert\NotBlank]
    private ?string $name = null;

    /**
     * @var string|null The path (complete name)
     */
    #[ORM\Column(type: Types::TEXT)]
    #[Gedmo\TreePath]
    #[Assert\NotBlank]
    private ?string $path = null;

    /**
     * Initialization
     */
    public function __construct () {
        $this->documents = new ArrayCollection();
    }

    /**
     * The owner
     *
     * @return User|null The owner
     */
    public function getOwner (): ?User {
        return $this->owner;
    }
    /**
     * Change the owner
     *
     * @param User $owner The new owner
     *
     * @return $this
     */
    public function setOwner (User $owner): static {
        $this->owner = $owner;

        return $this;
    }

    /**
     * The name
     *
     * @return string|null The name
     */
    public function getName (): ?string {
        return $this->name;
    }
    /**
     * Change the name
     *
     * @param string $name The new name
     *
     * @return $this
     */
    public function setName (string $name): self {
        $this->name = $name;

        return $this;
    }

    /**
     * The path (complete name)
     *
     * @return string|null The path (complete name)
     */
    public function getPath (): ?string
    {
        return $this->path;
    }
    /**
     * Change the path (complete name)
     *
     * @param string $path The new path (complete name)
     *
     * @return $this
     */
    public function setPath (string $path): static
    {
        $this->path = $path;

        return $this;
    }
}
